return function ($app, ws, $scope, $route, $mdDialog, toaster, $translate) {
	$scope.servers = [];

	function validate_server($$hashKey, values) {
		const duplicate = $scope.servers.find((server) => server.name == values.name);
		const authentication_methods = [ "", "plain", "chap" ];
		if (values.name.trim() == "") {
			return new Error("Server name is required");
		} else if (duplicate && duplicate.$$hashKey != $$hashKey) {
			return new Error("Server name must be unique");
		} else if (values.address.trim() == "") {
			return new Error("Server address is required");
		} else if ($$hashKey == "" && values.shared_secret.trim() == "") {
			return new Error("Shared secret is required");
		} else if (!authentication_methods.includes(values.authentication)) {
			return new Error("Invalid authentication method");
		}
		for (const vsa of values.vsas) {
			if (vsa.key.trim() == "") {
				continue;
			} else if (!vsa.key.match(/^\d+(-\d+)*$/)) {
				return new Error("Vendor-specific attribute formatted incorrectly");
			}
		}
	}

	$scope.add_server = () => {
		$mdDialog.show({
			templateUrl: `/templates/add_radius_server.html?${Date.now()}`,
			controller: function($scope, $mdDialog) {
				$scope.vsas = [{ key: "", value: "" }];
				$scope.close = $mdDialog.cancel;
				$scope.save = () => {
					const values = {
						name: ($scope.name ?? "").trim(),
						address: ($scope.address ?? "").trim(),
						authentication: $scope.authentication ?? "",
						shared_secret: ($scope.shared_secret ?? ""),
						vsas: ($scope.vsas ?? []),
					};
					const err = validate_server("", values);
					if (err) {
						toaster.error(err.message);
					} else {
						$mdDialog.hide(values);
					}
				};
				$scope.add_vsa = () => {
					$scope.vsas.push({ key: "", value: "" });
				};
				$scope.remove_vsa = ($$hashKey) => {
					$scope.vsas = $scope.vsas.filter((vsa) => vsa.$$hashKey != $$hashKey);
				};
			},
		}).then((values) => {
			$scope.servers.push(values);
		}, () => {});
	};

	$scope.edit_server = ($$hashKey) => {
		const server = $scope.servers.find((server) => server.$$hashKey == $$hashKey);
		$mdDialog.show({
			templateUrl: `/templates/edit_radius_server.html?${Date.now()}`,
			controller: function($scope, $mdDialog) {
				$scope.name = server.name;
				$scope.address = server.address;
				$scope.authentication = server.authentication;
				$scope.close = $mdDialog.cancel;
				$scope.vsas = [ ...server.vsas, { key: "", value: "" } ];
				$scope.save = () => {
					const values = {
						name: ($scope.name ?? "").trim(),
						address: ($scope.address ?? "").trim(),
						authentication: $scope.authentication ?? "",
						new_shared_secret: ($scope.new_shared_secret ?? ""),
						vsas: ($scope.vsas ?? []),
					};
					const err = validate_server($$hashKey, values);
					if (err) {
						toaster.error(err.message);
					} else {
						$mdDialog.hide(values);
					}
				};
				$scope.add_vsa = () => {
					$scope.vsas.push({ key: "", value: "" });
				};
				$scope.remove_vsa = ($$hashKey) => {
					$scope.vsas = $scope.vsas.filter((vsa) => vsa.$$hashKey != $$hashKey);
				};
			},
		}).then((values) => {
			server.name = values.name;
			server.address = values.address;
			server.authentication = values.authentication;
			server.vsas = values.vsas;
			if (values.new_shared_secret.trim() != "") {
				server.shared_secret = values.new_shared_secret;
			}
		}, () => {});
	};

	$scope.delete_server = ($$hashKey) => {
		$mdDialog.show({
			templateUrl: `/templates/remove_radius_server.html?${Date.now()}`,
			controller: function($scope, $mdDialog) {
				$scope.close = $mdDialog.cancel;
				$scope.confirm = () => {
					$mdDialog.hide({});
				};
			},
		}).then(() => {
			$scope.servers = $scope.servers.filter((server) => server.$$hashKey != $$hashKey);
		}, () => {});
	};

	$scope.submit = () => {
		ws.update_radius_servers($scope.servers).then(() => {
			toaster.success($translate.instant("global.saved"));
		});
	};
	$app.creating($scope, "footer.save");

	ws.radius_servers().then((servers) => {
		$scope.servers = servers.map((server) => {
			const vsas = [];
			for (const [key, value] of Object.entries(server.vsas)) {
				vsas.push({key, value});
			}
			server.vsas = vsas;
			return server;
		})
	});
};
